/*
+-------------------------------- DISCLAIMER ---------------------------------+
|                                                                             |
| This application program is provided to you free of charge as an example.   |
| Despite the considerable efforts of Euresys personnel to create a usable    |
| example, you should not assume that this program is error-free or suitable  |
| for any purpose whatsoever.                                                 |
|                                                                             |
| EURESYS does not give any representation, warranty or undertaking that this |
| program is free of any defect or error or suitable for any purpose. EURESYS |
| shall not be liable, in contract, in torts or otherwise, for any damages,   |
| loss, costs, expenses or other claims for compensation, including those     |
| asserted by third parties, arising out of or in connection with the use of  |
| this program.                                                               |
|                                                                             |
+-----------------------------------------------------------------------------+
*/

// 2.0.0.15

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <iostream>
#include <stdlib.h>

#include "../Common/helper_vfs.h"

#define ESCAPE	27


using namespace std;
using namespace vfd_euresys;


bool CheckForQuit()
{
  struct termios oldt, newt;
  int ch;
  int oldf;

  tcgetattr(STDIN_FILENO, &oldt);
  newt = oldt;
  newt.c_lflag &= ~(ICANON | ECHO);
  tcsetattr(STDIN_FILENO, TCSANOW, &newt);
  oldf = fcntl(STDIN_FILENO, F_GETFL, 0);
  fcntl(STDIN_FILENO, F_SETFL, oldf | O_NONBLOCK);

  ch = getchar();

  tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
  fcntl(STDIN_FILENO, F_SETFL, oldf);

  if(ch == ESCAPE)
  {
    return true;
  }

  return false;
}

void StreamAudio( AudioSource * pAudio)
{
    	// stop any possible previous acquisition
	pAudio->StopAcquisition();
	// start acquisition
    	pAudio->StartAcquisition();

	// did the user pres <esc> ?
    	while (!CheckForQuit()) 
	{
		// Get new audio data ( this call is blocking) 
		AUDIO_DATA d = pAudio->GetData();
		std::cout.write ( d.data, d.size );
		// let the driver re-use this data space
		pAudio->ReleaseData();
	}

	// stop acquisition
    	pAudio->StopAcquisition();

}

void DisplayUsage()
{
	std::cerr << "Usage" << std::endl;
	std::cerr << "=====" << std::endl;
	std::cerr << "audio2stream mount_point audioinput" << std::endl;
	std::cerr << "where" << std::endl;
	std::cerr << "mount_point is the path where the board has been mounted" << std::endl;
	std::cerr << "e.g.: /mnt/ux_0" << std::endl;
	std::cerr << "audioinput is the audio source input number range from 0 to max audio source -1" << std::endl;    
}



int main ( int argc, char *argv[] )
{
	int status = 0;
	AudioSource * pAudio;
	pAudio = NULL;
	
	// parse command line
	int asrc = 0;

	if ( argc < 2 )
	{
		DisplayUsage();
		return -1;
	}

	// get audio input
	if ( argc > 2 )
	{
		sscanf ( argv[2],"%u", &asrc );
	}

	try
	{
		// create access class
		pAudio = new AudioSource ( argv[1], asrc );
		// audio streaming
		StreamAudio ( pAudio );
	}
	catch (std::exception &e)
	{
        std::cerr << "--- EXCEPTION --- " << std::endl;
        std::cerr << e.what() << std::endl;
        std::cerr << std::endl;
        status = -1;
    }
	catch (vfd_euresys::vfd_Exception &e)
	{
        std::cerr << "--- EXCEPTION in VFD --- " << std::endl;
        std::cerr << e.what() << std::endl;
        std::cerr << std::endl;
        status = -1;
    }

	return status;
}
