/*
+-------------------------------- DISCLAIMER ---------------------------------+
|                                                                             |
| This application program is provided to you free of charge as an example.   |
| Despite the considerable efforts of Euresys personnel to create a usable    |
| example, you should not assume that this program is error-free or suitable  |
| for any purpose whatsoever.                                                 |
|                                                                             |
| EURESYS does not give any representation, warranty or undertaking that this |
| program is free of any defect or error or suitable for any purpose. EURESYS |
| shall not be liable, in contract, in torts or otherwise, for any damages,   |
| loss, costs, expenses or other claims for compensation, including those     |
| asserted by third parties, arising out of or in connection with the use of  |
| this program.                                                               |
|                                                                             |
+-----------------------------------------------------------------------------+
*/

// 2.0.0.15

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <iostream>
#include <stdlib.h>

#include "../Common/helper_vfs.h"


#define ESCAPE	27

using namespace std;
using namespace vfd_euresys;

bool CheckForQuit()
{
  struct termios oldt, newt;
  int ch;
  int oldf;

  tcgetattr(STDIN_FILENO, &oldt);
  newt = oldt;
  newt.c_lflag &= ~(ICANON | ECHO);
  tcsetattr(STDIN_FILENO, TCSANOW, &newt);
  oldf = fcntl(STDIN_FILENO, F_GETFL, 0);
  fcntl(STDIN_FILENO, F_SETFL, oldf | O_NONBLOCK);

  ch = getchar();

  tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
  fcntl(STDIN_FILENO, F_SETFL, oldf);

  if(ch == ESCAPE)
  {
    return true;
  }

  return false;
}

void RenderVideo ( VisualFormatter * source )
{
	// stop any possible previous acquisition
	source->StopAcquisition();
	// start acquisition
	source->StartAcquisition();

	// did the user pres <esc> ?
	while ( !CheckForQuit() )
	{
		// Get new data chunk ( this call is blocking)
		FORMATTED_DATA d = source->GetData();
		std::cout.write ( d.data, source->GetBufferSize() );
		// let the driver re-use this data space
		source->ReleaseData ( d );
	}

	// stop acquisition
	source->StopAcquisition();

};

// display the expected arguments

void DisplayUsage()
{
	std::cerr << "Usage" << std::endl;
	std::cerr << "=====" << std::endl;
	std::cerr << "raw2stream mount_point videoinput " << std::endl;
	std::cerr << "where" << std::endl;
	std::cerr << "mount_point is the path where the board has been mounted" << std::endl;
	std::cerr << "e.g.: /mnt/ux_0" << std::endl;
	std::cerr << "videoinput is the video source input number range from 0 to max video source -1" << std::endl;    

}

int main ( int argc, char *argv[] )
{
	int status = 0;


	int vsrc =0;
	if ( argc < 2 )
	{
		DisplayUsage();
		return -1;
	}

	if ( argc > 2 )
	{
		sscanf ( argv[2],"%u", &vsrc );
	}

	try
	{
		// create access class
		VisualFormatter mysource ( argv[1], vsrc );
		// Set a higher amount of buffer (optionnal)
		mysource.SetBufferCount ( 10 );
		// define some captions
		sprintf ( mysource.m_Caption[0].text, "vid %d",vsrc );
		mysource.m_Caption[0].pos = VisualSource::BOTTOMLEFT;

		sprintf ( mysource.m_Caption[1].text, "Raw Video" );
		mysource.m_Caption[1].X = 50;
		mysource.m_Caption[1].Y = 50;

		// write everything back
		// function call is mandatory as it also define
		// the video standard
		mysource.WriteElements();
		// Set Color format can be set any time when the acquisition is set to idle
		mysource.SetColorFormat ( YUV420PL );
		// continue rendering
		RenderVideo ( &mysource );
	}
	catch (std::exception &e)
	{
        std::cerr << "--- EXCEPTION --- " << std::endl;
        std::cerr << e.what() << std::endl;
        std::cerr << std::endl;
        status = -1;
    }
	catch (vfd_euresys::vfd_Exception &e)
	{
        std::cerr << "--- EXCEPTION in VFD --- " << std::endl;
        std::cerr << e.what() << std::endl;
        std::cerr << std::endl;
        status = -1;
    }

	return status;
}
