/*
+-------------------------------- DISCLAIMER ---------------------------------+
|                                                                             |
| This application program is provided to you free of charge as an example.   |
| Despite the considerable efforts of Euresys personnel to create a usable    |
| example, you should not assume that this program is error-free or suitable  |
| for any purpose whatsoever.                                                 |
|                                                                             |
| EURESYS does not give any representation, warranty or undertaking that this |
| program is free of any defect or error or suitable for any purpose. EURESYS |
| shall not be liable, in contract, in torts or otherwise, for any damages,   |
| loss, costs, expenses or other claims for compensation, including those     |
| asserted by third parties, arising out of or in connection with the use of  |
| this program.                                                               |
|                                                                             |
+-----------------------------------------------------------------------------+
*/
// 2.0.0.15

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <stdlib.h>
#include "SDL.h"

#include "../Common/helper_vfs.h"

using namespace std;
using namespace vfd_euresys;
bool CheckForQuit()
{
    SDL_Event event;

    // poll for events, and handle <esc> key
    while (SDL_PollEvent(&event)) {
        switch (event.type) {
        case SDL_KEYUP:
            // if escape is pressed, return (and thus, quit)
            if (event.key.keysym.sym == SDLK_ESCAPE)
                return true;
            break;
        case SDL_QUIT:
            return true;
        }
    }

    return false;
};

void RenderFrames(VisualFormatter * source)
{
    // create a SizeX * SizeY window with 32 bits per pixels.
    SDL_Surface *screen = SDL_SetVideoMode(source->GetSizeX(), source->GetSizeY(),
        32, SDL_SWSURFACE | SDL_DOUBLEBUF);
    if (screen == NULL) {
        std::cerr << "Unable to set video mode: " << SDL_GetError() << std::endl;
        return;
    }
	// did the user pres <esc> ?
    while (!CheckForQuit()) 
	{
		// Get new data chunk ( this call is blocking) 
	    FORMATTED_DATA d = source->GetData();
		// copy acquired data in screen
        if (SDL_MUSTLOCK(screen))
            SDL_LockSurface(screen);
        memcpy(screen->pixels, d.data, source->GetBufferSize());
        if (SDL_MUSTLOCK(screen))
            SDL_UnlockSurface(screen);
        // display data
		SDL_Flip(screen);
		// let the driver re-use this data space
        source->ReleaseData(d);
    }
};

void RenderVideo(VisualFormatter * source)
{
    // stop any possible previous acquisition
	source->StopAcquisition();
	// start acquisition
    source->StartAcquisition();
    // render each frame
	RenderFrames(source);
	// stop acquisition
    source->StopAcquisition();

};


int main(int argc, char *argv[])
{
    int status = 0;
    // Initialize SDL's video subsystem.
    if (SDL_Init(SDL_INIT_VIDEO) < 0) {
        std::cerr << "Unable to init SDL: " << SDL_GetError() << std::endl;
        return -1;
    }
    
    try {
		// create access class
 		VisualFormatter mysource("/mnt/ux_0", 0);
		// Set a higher amount of buffer (optionnal)
		mysource.SetBufferCount(10);
		// define some captions
		sprintf(mysource.m_Caption[0].text, "vid %d",1);
		mysource.m_Caption[0].pos = VisualSource::BOTTOMLEFT;

		sprintf(mysource.m_Caption[1].text, "Raw Video");
		mysource.m_Caption[1].X = 50;
		mysource.m_Caption[1].Y = 50;
		// define a privacy mask
		mysource.m_Mask[0].X = 50;
		mysource.m_Mask[0].Y = 85;
		mysource.m_Mask[0].W = 150;
		mysource.m_Mask[0].H = 10;
		// write everything back
		// function call is mandatory as it also define 
		// the video standard
		mysource.WriteElements();
		// Set Color format can be set any time when the acquisition is set to idle
		mysource.SetColorFormat(RGB32);
        // continue rendering
        RenderVideo(&mysource);
    } 
	catch (std::exception &e)
	{
        std::cerr << "--- EXCEPTION --- " << std::endl;
        std::cerr << e.what() << std::endl;
        std::cerr << std::endl;
        status = -1;
    }
	catch (vfd_euresys::vfd_Exception &e)
	{
        std::cerr << "--- EXCEPTION in VFD --- " << std::endl;
        std::cerr << e.what() << std::endl;
        std::cerr << std::endl;
        status = -1;
    }
    SDL_Quit();

    return status;
}
